//###################################################################################
// File: doswrite3.c
//
// 2MB groe Dateien auf den CF schreiben und wieder lesen.
// Jeweils mit unterschiedlichen Puffergren. Zeit messen und anzeigen.
//
//#########################################################################
// Last change: 11.03.2007
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.5
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "protos.h"
#include "mydefs.h"
#include "../dos.h"
#include "../printf.h"
#include "../serial.h"

#define TEST_FILE_SIZE	(unsigned long)2000 * (unsigned long)1024  // test filesize 2MB

void StartTimer(void);
void StopTimer(void);
void ReadTestFile(char *name, U16 bufsize);
void WriteTestFile(char *name, U16 bufsize);

//"volatile" fr alle Variablen die in Interrupt's gendert werden
unsigned long volatile countclock;
unsigned char buf[]="Hello World. Hello World. Hello World. Hello World. Hello World. Hello World. Hello World. Hello World. Hello World. Hello World. \n";

//###################################################################################
SIGNAL(SIG_OVERFLOW1)        // signal handler for timer1 overflow interrupt
//###################################################################################
{
// TCNT1 = 65535 - ((F_CPU / 64) / 1000 ); //Counter Reload for 1ms
 TCNT1 = 65535 - ((F_CPU / 64) / 100 ); //Counter Reload for 10ms
// TCNT1 = 65535 - ((F_CPU / 64) / 10 ); //Counter Reload for 100ms

 countclock++;
}


//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{
 DDRC=0xFF;  	// set io-pins on PortC
 PORTC=0; 	//LCD-Datenbus

 DDRD=0xF2;  	// set io-pins on PortD
 PORTD=0x02; 	//LCD-Steuersignale siehe lcd.h

 DDRA=0; 	//Alles Eingnge
 PORTA=0xFF; 	//Eingnge mit Pullups setzen

#if !defined (__AVR_ATmega161__) && !defined (__AVR_ATmega644__)
 SFIOR&=!(1<<PUD); 	//Pullups Enable
#else
#endif
// SFIOR=0; 	//Pullups Enable

#ifdef MMC_CARD_SPI //SD_CARD_SPI too !
 MMC_IO_Init();
#endif

 ser_init();

 sei();              //enable interrupts

 puts("ATMega FAT DOS-Read-Write-Test3\n");
 puts(" by Holgi\n");

 if(GetDriveInformation()!=F_OK) // get drive parameters
  {
   puts("No Flash !");
   while(1);
  }
  
 RED_ON();
 GREEN_OFF();

#ifdef USE_FAT12
 puts("Using FAT12\n");
#endif

#ifdef USE_FAT16
 puts("Using FAT16\n");
#endif

#ifdef USE_FAT32
 puts("Using FAT32\n");
#endif

#ifdef USE_FATBUFFER
 puts("\nUsing FAT Buffer\n");
#else
 puts("\nNo FAT Buffer\n");
#endif

#ifdef COMPACTFLASH_CARD
 puts("CF Card\n");
#endif
#ifdef MMC_CARD_SPI //SD_CARD_SPI too !
 puts("MMC/SD Card\n");
#endif

#ifdef STANDARD_SPI_WRITE  // No optimizations   
 puts("STANDARD_SPI_WRITE\n");
#endif
#ifdef FAST_SPI_WRITE      // Optimizations
 puts("FAST_SPI_WRITE\n");
#endif
#ifdef HYPER_SPI_WRITE      // Best Optimizations
 puts("HYPER_SPI_WRITE\n");
#endif
#ifdef STANDARD_SPI_READ  // No optimizations   
 puts("STANDARD_SPI_READ\n");
#endif
#ifdef FAST_SPI_READ      // Optimizations
 puts("FAST_SPI_READ\n");
#endif
#ifdef HYPER_SPI_READ      // Best Optimizations
 puts("HYPER_SPI_READ\n");
#endif


#ifdef SINGLE_BYTE_OPT_FWRITE
 puts("SINGLE_BYTE_OPT_FWRITE\n");
#endif
#ifdef FWRITE_SMALL_BUFFERS
 puts("FWRITE_SMALL_BUFFERS\n");
#endif
#ifdef SMALL_FWRITE   // No optimizations   
 puts("SMALL_FWRITE\n");
#endif
#ifdef FAST_FWRITE      // Optimizations
 puts("FAST_FWRITE\n");
#endif
#ifdef SINGLE_BYTE_OPT_FREAD
 puts("SINGLE_BYTE_OPT_FREAD\n");
#endif
#ifdef FREAD_SMALL_BUFFERS
 puts("FREAD_SMALL_BUFFERS\n");
#endif
#ifdef SMALL_FREAD  // No optimizations   
 puts("SMALL_FREAD\n");
#endif
#ifdef FAST_FREAD      // Optimizations
 puts("FAST_FREAD\n");
#endif

 printf("F_CPU %lu\n",F_CPU);

 Remove("01.txt"); //Remove last data
 Remove("02.txt");
 Remove("04.txt");
 Remove("08.txt");
 Remove("16.txt");
 Remove("32.txt");
 Remove("64.txt");
 Remove("128.txt");

 puts("\nStart writing files\n");
 WriteTestFile("01.TXT",1);
 WriteTestFile("02.TXT",2);
 WriteTestFile("04.TXT",4);
 WriteTestFile("08.TXT",8);
 WriteTestFile("16.TXT",16);
 WriteTestFile("32.TXT",32);
 WriteTestFile("64.TXT",64);
 WriteTestFile("128.TXT",128);

 puts("\nStart reading files\n");
 ReadTestFile("01.TXT",1);
 ReadTestFile("02.TXT",2);
 ReadTestFile("04.TXT",4);
 ReadTestFile("08.TXT",8);
 ReadTestFile("16.TXT",16);
 ReadTestFile("32.TXT",32);
 ReadTestFile("64.TXT",64);
 ReadTestFile("128.TXT",128);

 puts("Test done.\n");

 GREEN_ON();
 RED_OFF();


 for(;;) // loop forever
  {
  }
}

//###################################################################################
void WriteTestFile(char *name, U16 bufsize)
//###################################################################################
{
 unsigned long i;
 unsigned char result;

 result=Fopen(name,F_WRITE);
 if(result==F_OK)
  {
   StartTimer();
   for(i=0; i<TEST_FILE_SIZE; i+=bufsize)
    {
     if(Fwrite(buf,bufsize)!=bufsize) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   printf("% 3u",bufsize);
   StopTimer();
  }
}

//###################################################################################
void ReadTestFile(char *name, U16 bufsize)
//###################################################################################
{
 unsigned long i;
 unsigned char result;

 result=Fopen(name,F_READ);
 if(result==F_OK)
  {
   StartTimer();
   for(i=0; i<TEST_FILE_SIZE; i+=bufsize)
    {
     if(Fread(buf,bufsize)!=bufsize) result=F_ERROR;
     if(result==F_ERROR) break;
    }
   Fclose();
   printf("% 3u",bufsize);
   StopTimer();
  }
}


void StartTimer(void)
{
 countclock=0;
//Timer1 Configuration
// TCNT1 = 65535 - ((F_CPU / 64) / 1000 ); //Counter Reload for 1ms
 TCNT1 = 65535 - ((F_CPU / 64) / 100 ); //Counter Reload for 10ms
// TCNT1 = 65535 - ((F_CPU / 64) / 10 ); //Counter Reload for 100ms

 TCCR1A=0; 
 TCCR1B=((1<<CS11) | (1<<CS10) ); // prescaler 1/64

#if defined (__AVR_ATmega644__)
 TIMSK1|=(1<<TOIE1);   // enable timer1 int
#else
 TIMSK|=(1<<TOIE1);   // enable timer1 int
#endif
}

void StopTimer(void)
{
 unsigned long tmp1,tmp2;
 float kBps;

#if defined (__AVR_ATmega644__)
 TIMSK1 &= ~(1<<TOIE1);   // disable timer1 int
#else
 TIMSK &= ~(1<<TOIE1);   // disable timer1 int
#endif

/*
 LCDPos(1,1);
 printf("%lX ",FileCurrentCluster);

 LCDPos(1,10);
 printf("%lu ",FileCurrentSector);

 LCDPos(1,20);
 printf("%lu\n",FileSize);
*/

 tmp1 = countclock / 100; // timer ticks per count is 10ms, this is the seconds part
 tmp2 = countclock % 100; // timer tick counts below 1s 
 printf(" % 3lu.%02lus ",tmp1,tmp2);

// LCDPos(2,10);
 // We have to use float here to avoid rounding
 kBps = (float)TEST_FILE_SIZE / (float)countclock; //Bytes per 0.01 seconds
 kBps *= 1000.0; //Bytes per 10 seconds to get xxx.x values
 kBps /= 1024.0; // kiloBytes per 10 seconds
 tmp1 = (unsigned int)(kBps / 10.0);
 tmp2 = (unsigned int)kBps % 10;
 printf("% 3lu.%lukB/s\n",tmp1,tmp2);
}
